

#define F_CPU 16000000

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <stdlib.h>

#define normal_speed 32050
#define fast_speed  4012
#define delay 1

#define Max_top_offset 4
#define num_lines 8
#define form_type_ceilling 4
#define top_offset_ceilling 5
#define delay_make_end_life 200

#define poz0 0
#define poz1 1
#define poz2 2

#define step 3

volatile int  bot_form_type0 = 1;
volatile int  bot_form_type1 = 2;
volatile int bot_form_type2 = 3;
volatile int top_offset = 0;
volatile int  top_form_type0;
volatile int  top_form_type1;
volatile int top_form_type2;

volatile int score = 0;
volatile int speeds[] ={32050, 16025, 8012, 4012};
volatile int current_speed = 0;
volatile int next_speed = 1;


/*functie de initializare a randomului*/
void initrand()
{
        uint32_t state;
        static uint32_t EEMEM sstate;

        state = eeprom_read_dword(&sstate);

        // Check if it's unwritten EEPROM (first time). Use something funny
        // in that case.
        if (state == 0xffffffUL)
                state = 0xDEADBEEFUL;
        srandom(state);
        eeprom_write_dword(&sstate, random());
}

/*se genereaza datele random*/
/*formele figurilor*/
void get_random_data()
{
	bot_form_type0 = random() % form_type_ceilling;
	bot_form_type1 = random() % form_type_ceilling;
	bot_form_type2 = random() % form_type_ceilling;

	top_form_type0 = random() % form_type_ceilling;
	top_form_type1 = random() % form_type_ceilling;
	top_form_type2 = random() % form_type_ceilling;

}

/*initializare input/output*/
void init()
{
	/*PORTB linii*/
	DDRB |= (1 << PB0);
	DDRB |= (1 << PB1);
	DDRB |= (1 << PB2);
	DDRB |= (1 << PB3);
	DDRB |= (1 << PB4);
	DDRB |= (1 << PB5);
	DDRB |= (1 << PB6);
	DDRB |= (1 << PB7);

	/*PORTA coloane*/
	DDRA |= (1 << PA0);
	DDRA |= (1 << PA1);
	DDRA |= (1 << PA2);
	DDRA |= (1 << PA3);
	DDRA |= (1 << PA4);
	DDRA |= (1 << PA5);
	DDRA |= (1 << PA6);
	DDRA |= (1 << PA7);

	/*cele patru butoane cu rezistent de pull-up*/
	DDRC &= ~(1 << PC0);
	PORTC |= (1 << PC0);
	DDRC &= ~(1 << PC1);
	PORTC |= (1 << PC1);
	DDRC &= ~(1 << PC6);
	PORTC |= (1 << PC6);
	DDRC &= ~(1 << PC7);
	PORTC |= (1 << PC7);


}

/*initializarea timerului*/
void init_timer()
{
	// pragul initial de comparare
	OCR1A = normal_speed;
	// activeaza intreruperea de comparare cu OCR1A
	TIMSK1 = (1 << OCIE1A);
	// modul CTC cu OCR1A
	// prescaler 1024
	TCCR1B = (5 << CS10) | (1 << WGM12);
}

/*se dezactiveaza fiecare LED*/
/*este apelata din intrerupere*/
void make_clean()
{
	int i;
	for ( i = 0; i < num_lines; i++)
	{
		PORTA &= ~(1 << (PA0 + i));
		PORTB &= ~(1 << (PB0 + i));
	}
}

/*efectul de sfarsit*/
void make_end_life()
{
	int i;
	for ( i = 0; i < num_lines; i++)
	{
		PORTB  |= (1 << (PB0 + i));
		PORTA  |= (1<< (PA0 + i));
			_delay_ms(delay_make_end_life);
		/*resetez timerul viteza si scorul*/
		TCNT1 = 0;
		current_speed = 0;
		next_speed = 1;
		OCR1A = speeds[current_speed];
		score = 0;
	}
}

/*verificare daca formele de sus sunt la fel ca cele de jos*/
int check()
{
		return (top_form_type0 == bot_form_type0 && top_form_type1 == bot_form_type1 && top_form_type2 == bot_form_type2);
}

/*rutina de tratare intreruperii generate de timer*/
/*comparare cu OCR1A*/
ISR(TIMER1_COMPA_vect)
{
		int ret;

		/*daca formele de sus sunt lipite de cele de jos*/
		if (top_offset == Max_top_offset)
		{
			/*verific daca sunt la fel*/
			ret = check();
			if(ret)
			{
				/*incep o noua runda*/
				score++;
				/*dupa 3 runde terminate cu succes cresc viteza*/
				if (score % step == 0 && speeds[current_speed] != fast_speed)
				{
					OCR1A = speeds[next_speed];
					current_speed = next_speed;
					next_speed++;
				}
				get_random_data();
			}
		else
			{
				/*efectul de sfarsit si inceperea unei noi runde*/
				make_clean();
				make_end_life();
				make_clean();
				get_random_data();
			}
		}
		/*incrementarea offsetului*/
		/*lasarea in jos a formelor de sus*/
    top_offset = (top_offset + 1) % top_offset_ceilling;
		/*restarea timerului*/
		TCNT1 = 0;
}

/*afisarea formelor de jos*/
void make_bot_form(int type, int poz)
{
	int offset = -1;

	/*in functie de pozitie se afla offsetul pe coloane*/
	switch (poz) {
		case poz0:
			offset = 0;
			break;
		case poz1:
			offset = 3;
			break;
		case poz2:
			offset = 6;
			break;
		default:
			break;
	}

	/*in functie de tip se afiseaza forma*/
	switch (type) {
		/*doar un singur led*/
		case 0:
			PORTA |= (1 << (PA0 + offset));
			PORTB |= (1 << PB7);
			_delay_ms(delay);
			PORTA &= ~(1 << (PA0 + offset));
			PORTB &= ~(1 << PB7);
			break;
		/*doua leduri*/
		case 1:
			PORTA |= (1 << (PA0 + offset));
			PORTB |= (1 << PB7);
			_delay_ms(delay);
			PORTA &= ~(1 << (PA0 + offset));
			PORTB &= ~(1 << PB7);

			PORTA |= (1 << (PA0 + offset));
			PORTB |= (1 << PB6);
			_delay_ms(delay);
			PORTA &= ~(1 << (PA0 + offset));
			PORTB &= ~(1 << PB6);
			break;
		/*trei leduri*/
		case 2:
			PORTA |= (1 << (PA0 + offset));
			PORTB |= (1 << PB7);
			_delay_ms(delay);
			PORTA &= ~(1 << (PA0 + offset));
			PORTB &= ~(1 << PB7);

			PORTA |= (1 << (PA0 + offset));
			PORTB |= (1 << PB6);
			_delay_ms(delay);
			PORTA &= ~(1 << (PA0 + offset));
			PORTB &= ~(1 << PB6);

			PORTA |= (1 << (PA1 + offset));
			PORTB |= ( 1 << PB7);
			_delay_ms(delay);
			PORTA &= ~(1 << (PA1 + offset));
			PORTB &= ~( 1 << PB7);
			break;

		/*4 leduri*/
		case 3:
			PORTA |= (1 << (PA0 + offset));
			PORTB |= (1 << PB7);
			_delay_ms(delay);
			PORTA &= ~(1 << (PA0 + offset));
			PORTB &= ~(1 << PB7);

			PORTA |= (1 << (PA0 + offset));
			PORTB |= (1 << PB6);
			_delay_ms(delay);
			PORTA &= ~(1 << (PA0 + offset));
			PORTB &= ~(1 << PB6);

			PORTA |= (1 << (PA1 + offset));
			PORTB |= ( 1 << PB7);
			_delay_ms(delay);
			PORTA &= ~(1 << (PA1 + offset));
			PORTB &= ~( 1 << PB7);

			PORTA |= (1 << (PA1 + offset));
			PORTB |= ( 1 << PB6);
			_delay_ms(delay);
			PORTA &= ~(1 << (PA1 + offset));
			PORTB &= ~( 1 << PB6);
			break;

		default:
			break;
		}


}

/*formele de sus*/
/*analog ca cele de jos*/
/*al treilea parametru reprezinta distanta fata de prima linie*/
void make_top_form(int type, int poz, int height)
{
	int offset = -1;

	switch (poz) {
		case poz0:
			offset = 0;
			break;
		case poz1:
			offset = 3;
			break;
		case poz2:
			offset = 6;
			break;
		default:
			break;
	}
	switch (type) {
		case 0:
			PORTA |= (1 << (PA0 + offset));
			PORTB |= (1 << (PB0 + height));
			_delay_ms(delay);
			PORTA &= ~(1 << (PA0 + offset));
			PORTB &= ~(1 << (PB0 + height));
			break;

		case 1:
			PORTA |= (1 << (PA0 + offset));
			PORTB |= (1 << (PB0 + height));
			_delay_ms(delay);
			PORTA &= ~(1 << (PA0 + offset));
			PORTB &= ~(1 << (PB0 + height));

			PORTA |= (1 << (PA0 + offset));
			PORTB |= (1 << (PB0 + height + 1));
			_delay_ms(delay);
			PORTA &= ~(1 << (PA0 + offset));
			PORTB &= ~(1 << (PB0 + height + 1));
			break;

		case 2:
			PORTA |= (1 << (PA0 + offset));
			PORTB |= (1 << (PB0 + height));
			_delay_ms(delay);
			PORTA &= ~(1 << (PA0 + offset));
			PORTB &= ~(1 << (PB0 + height));

			PORTA |= (1 << (PA1 + offset));
			PORTB |= ( 1 << (PB0 + height + 1));
			_delay_ms(delay);
			PORTA &= ~(1 << (PA1 + offset));
			PORTB &= ~( 1 << (PB0 + height + 1));

			PORTA |= (1 << (PA0 + offset));
			PORTB |= (1 << (PB0 + height + 1));
			_delay_ms(delay);
			PORTA &= ~(1 << (PA0 + offset));
			PORTB &= ~(1 << (PB0 + height + 1));
			break;


		case 3:
			PORTA |= (1 << (PA0 + offset));
			PORTB |= (1 << (PB0 + height));
			_delay_ms(delay);
			PORTA &= ~(1 << (PA0 + offset));
			PORTB &= ~(1 << (PB0 + height));

			PORTA |= (1 << (PA0 + offset));
			PORTB |= (1 << (PB0 + height + 1));
			_delay_ms(delay);
			PORTA &= ~(1 << (PA0 + offset));
			PORTB &= ~(1 << (PB0 + height + 1));

			PORTA |= (1 << (PA1 + offset));
			PORTB |= ( 1 << (PB0 + height + 1));
			_delay_ms(delay);
			PORTA &= ~(1 << (PA1 + offset));
			PORTB &= ~( 1 << (PB0 + height + 1));

			PORTA |= (1 << (PA1 + offset));
			PORTB |= ( 1 << (PB0 + height));
			_delay_ms(delay);
			PORTA &= ~(1 << (PA1 + offset));
			PORTB &= ~( 1 << (PB0 + height));
			break;

		default:
			break;
	}
}
/*functie de afisare a tuturor formelor*/
void display()
{
	make_bot_form(bot_form_type0, poz0);
	make_bot_form(bot_form_type1, poz1);
	make_bot_form(bot_form_type2, poz2);
	make_top_form(top_form_type0, poz0, top_offset);
	make_top_form(top_form_type1, poz1, top_offset);
	make_top_form(top_form_type2, poz2, top_offset);
}
int main(void) {

	/*initializari*/
	init();
	init_timer();
	initrand();
	get_random_data();
	sei();

	/*bucla principala*/
	while(1)
	{
		/*apasare buton de viteza*/
		if( (PINC & (1 << PC7)) == 0 )
		{
				TCNT1 = OCR1A % fast_speed;
				OCR1A = fast_speed;
				while ((PINC & (1 << PC7)) == 0)
				{
					display();
				}
				OCR1A = speeds[current_speed];
		}

		/*apasare buton pentru prima forma*/
		if((PINC & (1 << PC0)) == 0)
		{
				bot_form_type2 = (bot_form_type2 + 1)  % form_type_ceilling;
				while ((PINC & (1 << PC0)) == 0)
				{
							display();
				}
		}

		/*a doua forma*/
		if((PINC & (1 << PC1)) == 0)
		{
					bot_form_type0 = (bot_form_type0 + 1)  % form_type_ceilling;
					while ((PINC & (1 << PC1)) == 0)
					{
						display();
					}
		}
		/*a treia forma*/
		if((PINC & (1 << PC6)) == 0)
		{
					bot_form_type1 = (bot_form_type1+ 1)  % form_type_ceilling;
					while ((PINC & (1 << PC6)) == 0)
					{
						display();
					}
		}
				display();
	}

	return 0;
}
